/* temperature.h -- Temperature Monitor Interface
   Copyright 2001, 2002 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of EBCS.

EBCS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

EBCS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with EBCS; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#ifndef _EBCS_TEMPERATURE_H
#define _EBCS_TEMPERATURE_H

#include <sys/param.h>
#include <gel/event.h>
#include <gel/timer.h>

#ifdef __cplusplus
extern "C" {
#endif

/*! @defgroup temperature Temperature Monitor Interface

    This module monitors the temperature captors of EBCS.  There are
    two temperature captors, one for the room and one for the bath.
    The two captors are connected to the HC11 analog inputs through an
    operational amplifier.  The two operational amplifiers do not have
    the same gain.  The two captors are LM35DZ captors (10mv per degree C).

    The monitor collects the temperature on a regular basis.  It computes
    an average over some longer delay.  For this, the monitor has two timers.
    A first timer is used to collect the temperature by reading the analog
    port.  The second timer computes the average and keep it in the
    sample table.

 */
/*@{*/

typedef unsigned short temperature_t;

#define NR_TEMP_SAMPLES 32

/*! Statistics about the triac (ie, device ON/OFF stats).  */
struct temperature_monitor
{
  unsigned short* factor;
  unsigned char   port;
  temperature_t   samples[NR_TEMP_SAMPLES];
  struct timer    timer_meas;
  struct timer    timer_samples;
  temperature_t   *sample_last;
  temperature_t   temp_current;
  unsigned short  meas_period;
  unsigned short  sample_period;
};
typedef struct temperature_monitor temperature_monitor_t;


/*! Return the current temperature.

    Return the current temperature measured by the monitor.
    The value returned is the average over the last period.

    @param mon   Temperature monitor object
    @return      The temperature average in tens of degree C
*/
extern unsigned get_temperature (temperature_monitor_t *mon);
  
extern void create_temperature_monitor (temperature_monitor_t *tmon,
                                        unsigned char port,
                                        unsigned short *factor);

extern void temperature_initialize (void);

/*@}*/

#ifdef __cplusplus
};
#endif
#endif
