/* triacs.h -- Triacs Control Interface
   Copyright 2001, 2002 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of EBCS.

EBCS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

EBCS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with EBCS; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#ifndef _EBCS_TRIACS_H
#define _EBCS_TRIACS_H

#include <sys/param.h>
#include <gel/event.h>
#include <gel/timer.h>

#ifdef __cplusplus
extern "C" {
#endif

/*! @defgroup triacs Triacs Control Interface

  The triacs module controls the triacs of EBCS board.  There are
  two triacs, one for the light and one for the fan.  They are
  connected to port A, bit 5 and bit 6.  The module exports two
  operations:

  <ul>
  <li>triacs_set

  <li>triacs_get
  </ul>

  For each triac, it exports an event that is triggered when the
  triac state changes.  It also maintains statistics about the
  device usage such as the number of times it is switched on and
  the total time it was on.

 */
/*@{*/

/*! Type identifying the triac.  */
typedef enum triac_id
{
  /*! Triac controlling the light.  */
  TRIAC_LIGHT = 0,

  /*! Triac controlling the fan.  */
  TRIAC_FAN = 1
} triac_id_t;


/*! Type defining the state of a triac.  */
typedef enum triac_state
{
  /*! Triac is OFF.  */
  TRIAC_OFF = 0,

  /*! Triac is ON.  */
  TRIAC_ON = 1
} triac_state_t;


/*! Statistics about the triac (ie, device ON/OFF stats).  */
struct triac_stat
{
  /*! Number of times the triac was switched ON.  */
  unsigned long  on_counts;

  /*! Time when the triac was switched ON.  */
  struct timeval on_time;

  /*! Total time the triac was switched ON.  */
  struct timeval on_total_time;
};
typedef struct triac_stat triac_stat_t;


/*! Turn on or off the given triac.

  The triac identified by \i id is turned ON or OFF according to
  \i state.  If the triac state changes, an event is posted on
  the corresponding triac event object.

  @param id the triac identification
  @param state the new state of the triac (ON or OFF)

  @see triac_get, triac_events  */
extern void triac_set (triac_id_t id, triac_state_t state);

/*! Get the current state of the triac.

  Returns the state of the triac identified by \i id.

  @param id the triac identification
  @see triac_set  */
extern triac_state_t triac_get (triac_id_t id);

/*! Table of triac events.

  There is one event for each triac.  The event is triggered when the
  state of the triac is changed.

  @see triac_set, triac_get, event_post
*/
extern event_type triac_events[];

/*! Table of triac statistics.  */
extern triac_stat_t triac_stats[];

/*@}*/

#ifdef __cplusplus
};
#endif
#endif
