/* date-panel.c -- Date and Time Management Panels
   Copyright 2001 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <stddef.h>
#include <stdio.h>
#include <time.h>

#include <sys/locks.h>
#include <sys/ports.h>
#include <sys/sio.h>
#include <sys/interrupts.h>
#include <lcd.h>

#include <gel/timer.h>
#include <gel/event.h>
#include <gel/buttons.h>

#include <ebcs/ebcs.h>
#include <ebcs/board.h>
#include <ebcs/date-panel.h>

enum date_field {
  F_HOURS,
  F_MINS,
  F_SECS,
  F_WDAY,
  F_DAY,
  F_MONTH,
  F_YEAR
};

#define FIELD_COLUMN(N) ((N) & 0x0F)
#define FIELD_LINE(N)   ((N) >> 7)
#define FIELD_LENGTH(N) ((((N) >> 4) & 0x07) + 1)
#define FIELD(TYPE,LINE,COL,LEN) \
(((LINE) << 7) | ((COL) & 0x0F) | ((((LEN) - 1) & 0x07) << 4))

/*
  0123456789ABCDEF
  Lun 23  12:23:00
  Septembre 2001
*/
static const char fields[] = {
  FIELD (F_HOURS, 0,  8, 2), /* hours */
  FIELD (F_MINS,  0, 11, 2), /* mins */
  FIELD (F_SECS,  0, 14, 2), /* secs */
  FIELD (F_WDAY,  0,  0, 3), /* wday */
  FIELD (F_DAY,   0,  4, 2), /* day */
  FIELD (F_MONTH, 1,  0, 9),
  FIELD (F_YEAR,  1, 10, 4)
};

#define L_HOURS FIELD_LINE (fields[F_HOURS])
#define C_HOURS FIELD_COLUMN (fields[F_HOURS])
#define L_MINS  FIELD_LINE (fields[F_MINS])
#define C_MINS  FIELD_COLUMN (fields[F_MINS])
#define L_SECS  FIELD_LINE (fields[F_SECS])
#define C_SECS  FIELD_COLUMN (fields[F_SECS])
#define L_DAY   FIELD_LINE (fields[F_DAY])
#define C_DAY   FIELD_COLUMN (fields[F_DAY])
#define L_WDAY  FIELD_LINE (fields[F_WDAY])
#define C_WDAY  FIELD_COLUMN (fields[F_WDAY])
#define L_MONTH FIELD_LINE (fields[F_MONTH])
#define C_MONTH FIELD_COLUMN (fields[F_MONTH])
#define L_YEAR  FIELD_LINE (fields[F_YEAR])
#define C_YEAR  FIELD_COLUMN (fields[F_YEAR])

/* Forward declarations.  */
static void date_input (struct panel *p, enum key_mode mode, key_t key);
static void date_enter (struct panel *p);
static void date_setup (struct panel *p, enum date_field mode);
static void date_update (long delta);
static void date_putvalue (struct panel *p, lcd_line_t line,
                           lcd_col_t col, unsigned int value,
                           int need_all_digits);
static unsigned long date_refresh (struct panel *p);

static int cur_field;
static long increment;

const char *const day_names[] = {
  "Dimanche",
  "Lundi",
  "Mardi",
  "Mercredi",
  "Jeudi",
  "Vendredi",
  "Samedi"
};

const char *const month_names[] = {
  "Janvier",
  "Fevrier",
  "Mars",
  "Avril",
  "Mai",
  "Juin",
  "Juillet",
  "Aout",
  "Septembre",
  "Octobre",
  "Novembre",
  "Decembre"
};

struct panel ebcs_date_panel = {
  date_enter,
  0,
  date_refresh,
  date_input,
  0,
  10,
  0
};

static void
date_enter (struct panel *p)
{
  date_setup (p, F_HOURS);
  panel_show_cursor (p, 1);

  panel_putstring (p, L_HOURS, C_HOURS, "00:00:00");
}

void
date_putvalue (struct panel *p, lcd_line_t line, lcd_col_t col,
               unsigned int value, int need_all_digits)
{
  char c;

  c = (value / 10);
  if (c == 0 && need_all_digits == 0)
    c = ' ';
  else
    c = c + '0';

  panel_putchar (p, line, col, c);
  c = (value % 10) + '0';
  panel_putchar (p, line, col + 1, c);
}

void
date_putday (struct panel *p, lcd_line_t line, lcd_col_t col, struct tm *t)
{
  panel_putitem (p, line, col, JUSTIFY_LEFT, 3, day_names[t->tm_wday]);
  panel_putchar (p, line, col + 3, ' ');
  date_putvalue (p, line, col + 4, t->tm_mday, 0);
}

void
date_puttime (struct panel *p, lcd_line_t line, lcd_col_t col, struct tm *t)
{
  date_putvalue (p, line, col, t->tm_hour, 0);
  panel_putchar (p, line, col + 2, 'h');
  date_putvalue (p, line, col + 3, t->tm_min, 1);
  panel_putchar (p, line, col + 3 + 2, ':');
  date_putvalue (p, line, col + 3 + 3, t->tm_sec, 1);
}

static unsigned long
date_refresh (struct panel *p)
{
  struct timeval tv;
  struct timeval tv_end;
  struct tm t;
  unsigned year;

  timer_gettime (&tv);
  localtime_r (&tv.tv_sec, &t);
  date_puttime (p, L_HOURS, C_HOURS, &t);
  date_putday (p, L_WDAY, C_WDAY, &t);
  panel_putitem (p, L_MONTH, C_MONTH, JUSTIFY_LEFT, 10,
                 month_names[t.tm_mon]);

  year = t.tm_year + 1900;
  date_putvalue (p, L_YEAR, C_YEAR, year / 100, 1);
  date_putvalue (p, L_YEAR, C_YEAR + 2, year % 100, 1);

  timer_gettime (&tv_end);
  if (tv.tv_sec != tv_end.tv_sec)
    return 10L;
  else
    return (1000000L - tv_end.tv_usec) / 1000L;
}

static void
date_setup (struct panel *p, enum date_field mode)
{
  struct timeval tv;
  struct tm t;

  cur_field = mode;
  switch (mode)
    {
    case F_HOURS:
      increment = 3600;
      break;

    case F_MINS:
      increment = 60;
      break;

    case F_DAY:
      increment = 3600L * 24L;
      break;

    case F_YEAR:
      timer_gettime (&tv);
      localtime_r (&tv.tv_sec, &t);
      increment = 3600L * 24L * (long) t.tm_year_length;
      break;

    case F_MONTH:
      timer_gettime (&tv);
      localtime_r (&tv.tv_sec, &t);
      increment = 3600L * 24L * (long) t.tm_mon_length;
      break;

    case F_SECS:
      increment = 1;
      break;
    }

  panel_setcursor (p, FIELD_LINE (fields[cur_field]),
                   FIELD_COLUMN (fields[cur_field])
                   + FIELD_LENGTH (fields[cur_field]) - 1);
}

static void
date_update (long delta)
{
  struct timeval tv;

  timer_gettime (&tv);
  tv.tv_sec += delta;
  if (tv.tv_sec < 0)
    tv.tv_sec = 0;
  timer_settime (&tv);
}

static void
date_input (struct panel *p, enum key_mode mode, key_t key)
{
  if (mode == KEY_RELEASE && key == EBCS_KEY_PREV)
    {
      panel_pop ();
      return;
    }
  if (mode == KEY_PRESS && key == EBCS_KEY_NEXT)
    {
      switch (cur_field)
        {
        default:
        case F_HOURS:
          date_setup (p, F_MINS);
          break;

        case F_MINS:
          date_setup (p, F_SECS);
          break;

        case F_SECS:
          date_setup (p, F_DAY);
          break;

        case F_DAY:
          date_setup (p, F_MONTH);
          break;

        case F_MONTH:
          date_setup (p, F_YEAR);
          break;

        case F_YEAR:
          date_setup (p, F_HOURS);
          break;
        }
      return;
    }
  if (mode == KEY_PRESS && key == EBCS_KEY_PLUS)
    {
      date_update (increment);
      date_refresh (p);
    }
  if (mode == KEY_PRESS && key == EBCS_KEY_MINUS)
    {
      date_update (-increment);
      date_refresh (p);
    }
}

void
date_panel_initialize ()
{
  struct timeval tv;
  
  panel_create (&ebcs_date_panel);

  timer_gettime (&tv);
  if (tv.tv_sec < (365L * 24L * 3600L * (2002L - 1970L)) + 24L * 3600L * 10L)
    {
      tv.tv_sec = (365L * 24L * 3600L * (2002L - 1970L)) + 24L * 3600L * 10L;
      timer_settime (&tv);
    }
}

