/* debug.c -- Debug Agent for GEL
   Copyright 2001 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <sys/locks.h>
#include <sys/ports.h>
#include <sys/sio.h>
#include <sys/interrupts.h>
#include <stddef.h>

static void put_hex8 (unsigned short value);
static void put_hex16 (unsigned short value);
static void put_char(unsigned char c);
static void put_dec (unsigned short value);
static void put_str (const char *str);
static unsigned char get_char (void);
static long get_hex_short (unsigned char *p);
static void dump_memory (unsigned char *addr);

typedef struct rti_regs
{
  unsigned char  ccr;
  unsigned short d;
  unsigned short x;
  unsigned short y;
  unsigned short pc;
} rti_regs_t;

interrupt_t _debug_user_vectors[MAX_VECTORS];

static void
put_str (const char* str)
{
  while (*str)
    put_char (*str++);
}

static void
print_reg (unsigned short value, const char *name, int dump)
{
  put_str (name);
  put_char ('\t');
  put_hex16 (value);
  put_char ('\t');
  put_dec (value);
  if (dump)
    {
      put_char ('\t');
      dump_memory ((char *) value);
    }
  else
    put_char ('\n');
}

static void
dump_regs (struct rti_regs *regs)
{
  print_reg (regs->pc, "PC", 1);
  print_reg ((unsigned short) regs, "SP", 1);
  print_reg (regs->y, "Y", 1);
  print_reg (regs->x, "X", 1);
  print_reg (regs->d, "D", 1);
  print_reg (regs->ccr, "CCR", 0);
}

static void
dump_memory (unsigned char *addr)
{
  unsigned char i;

  put_hex16 ((unsigned short) addr);
  put_char ('\t');
  for (i = 0; i < 16; i++)
    {
      put_hex8 ((unsigned short) *addr);
      addr++;
      put_char (' ');
    }
  addr -= 16;
  put_char ('\t');
  for (i = 0; i < 16; i++)
    {
      unsigned char c;

      c = *addr++;
      if (c < ' ')
        c = ' ';
      else if (c & 0x80)
        c = '.';
      
      put_char (c);
    }
  put_char ('\n');
}

static void
debug_help ()
{
  put_str ("GEL Debugger v0.3 (c) 2001 Stephane Carrez\n");
  put_str ("Commands:\n\n");
  put_str ("?\t\tPrint registers\n");
  put_str ("C\t\tContinue\n");
  put_str ("R ADDR\t\tDump memory at addr (in hex)\n");
  put_str ("W ADDR\t\tWrite memory at addr (in hex)\n");
  put_str ("H\t\tPrint help\n");
}

#define MAX_LENGTH 16

static void
get_line (char *buf)
{
  unsigned char c;
  unsigned char len = 0;

  while (1)
    {
      c = get_char ();
      if (c == '\n' || c == '\r')
        {
          put_str ("\r\n");
          break;
        }

      if (c == '\b')
        {
          if (len > 0)
            {
              put_str ("\b \b");
              len--;
              buf--;
            }
        }
      else if (len < MAX_LENGTH)
        {
          if (c >= 'a' && c <= 'z')
            c = c + 'A' - 'a';

          put_char (c);
          *buf++ = c;
          len++;
        }
    }
  *buf = 0;
}

void
dbg_loop (unsigned char vector_id, struct rti_regs *regs)
{
  char buf[16];
  static int printing_on_lcd = 0;
  
  vector_id >>= 1;
  put_str ("$Exception$ ");
  put_dec ((unsigned short) vector_id);

  if (printing_on_lcd == 0)
    {
      printing_on_lcd = 1;
      lcd_print_at (0, 0, "GEL panic !!!!");
      lcd_print_at (0, 1, "System stopped ");
    }
  while (1)
    {
      char *p;
      char c;

      put_str ("\nDBG> ");
      get_line (buf);
      p = buf;
      do {
        c = *p++;
      } while (c && c <= ' ');
      if (c == 0)
        {
          debug_help ();
          continue;
        }

      /* Debug query state command.  Tell the debugger in which state we are.
         The vector_id indicates the exception that caused the debug agent to
         be called.  The user's stack pointer is passed so that registers
         can be fetched or modified by reading or writing the memory.  */
      if (c == '?')
        {
          dump_regs (regs);
          put_char ('\n');
        }

      /* Debug continue command.  Just exit the debug agent.
         The debugger must have changed the user's registers if it
         wanted to go to some other place.  */
      else if (c == 'C')
        {
          put_str ("$Resume$\n");
          break;
        }

      /* Debug read memory command.  */
      else if (c == 'R')
        {
          long addr = get_hex_short (p);

          if (addr < 0)
            put_str ("Invalid address\n");
          else
            dump_memory ((unsigned char*) ((unsigned short)  addr));
        }

      /* Debug write memory command.  */
      else if (c == 'W')
        {
          long addr = get_hex_short (p);

          if (addr < 0)
            put_str ("Invalid address\n");
          else
            {
              put_str (" Value> ");
              get_line (buf);
              p = buf;
              do {
                c = *p++;
              } while (c && c <= ' ');
              if (c)
                {
                  unsigned short val;
                  
                  p--;
                  val = get_hex_short (p);
                  p = (unsigned char*) ((unsigned short) addr);
                  *p = val;
                }
            }
        }

      else
        {
          debug_help ();
        }
    }
  printing_on_lcd = 0;
  _panel_force_refresh ();
}

/* Translate and print the 8-bit value in hexadecimal on the serial line.  */
static void
put_hex8 (unsigned short value)
{
  put_char ("0123456789ABCDEF"[(value >> 4) & 0x0F]);
  put_char ("0123456789ABCDEF"[value & 0x0F]);
}

/* Translate and print the 8-bit value in hexadecimal on the serial line.  */
static void
put_hex16 (unsigned short value)
{
  put_hex8 (value >> 8);
  put_hex8 (value);
}

static void
put_dec (unsigned short value)
{
  char buf[8];
  char* p;
  unsigned char c;
  
  p = &buf[8];
  *--p = 0;
  
  do {
    c = value % 10;
    value = value / 10;
    *--p = c + '0';
  } while (value != 0);
  put_str (p);
}

/* Write a character on the serial line.  */
static void
put_char (unsigned char c)
{
  while (!(_io_ports[M6811_SCSR] & M6811_TDRE))
    continue;

  _io_ports[M6811_SCDR] = c;
  _io_ports[M6811_SCCR2] |= M6811_TE;
}

static unsigned char
get_char (void)
{
  unsigned char c;
  volatile unsigned char* ports = &_io_ports[0];
  
  while (1)
    {
      c = ports[M6811_SCSR];
      if (c & M6811_RDRF)
        break;
    }
  return ports[M6811_SCDR];
}

static long
get_hex_short (unsigned char *buf)
{
  unsigned char c;
  unsigned short value;

  do {
    c = *buf++;
  } while (c && c <= ' ');
  buf--;
  value = 0;
  while ((c = *buf++))
    {
      if (c >= '0' && c <= '9')
        c = c - '0';
      else if (c >= 'a' && c <= 'f')
        c = c - 'a' + 10;
      else if (c >= 'A' && c <= 'F')
        c = c - 'A' + 10;
      else
        return -1L;

      value = (value << 4) | ((unsigned) c);
    }
  return (long) value;
}

void
abort ()
{
  asm ("swi");
}

void
exit (int code)
{
  asm ("swi");
}
