/* info-panel.c -- Info panel 
   Copyright 2001 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <stddef.h>
#include <stdio.h>
#include <sys/locks.h>
#include <sys/ports.h>
#include <sys/sio.h>
#include <sys/interrupts.h>
#include <lcd.h>

#include <gel/timer.h>
#include <gel/panel.h>
#include <gel/var.h>

#include <ebcs/ebcs.h>
#include <ebcs/board.h>
#include <ebcs/main-panel.h>
#include <ebcs/date-panel.h>
#include <ebcs/info-panel.h>

#define EBCS_KEY_ON EBCS_KEY_PLUS

static struct timer idle_timer;
static unsigned short cpu_usage;
static unsigned long cpu_idle;

/* Forward declarations.  */
static void ebcs_info_input (struct panel *p, enum key_mode mode, key_t key);
static unsigned long ebcs_info_refresh (struct panel *p);
static void idle_timer_handler (struct timer *t);

enum info_type
{
  INFO_FIRST = 0,
  INFO_VERSION = INFO_FIRST,
  INFO_CPU_USAGE,
#ifdef STACK_INFO
  INFO_STACK_SIZE,
#endif
  INFO_LAST
};

static int info_mode = INFO_FIRST;
extern struct panel ebcs_var_panel;

struct panel ebcs_info_panel = {
  0,
  0,
  ebcs_info_refresh,
  ebcs_info_input,
  0,
  10,
  0
};

unsigned short idle_loop_time = 150;

static const struct var_def idle_loop_var =
{
  "IDLE loop time",
  V_UINT16,
  F_INTEGER,
  10,
  900,
  150,
  { (unsigned char*) &idle_loop_time }
};

#ifdef STACK_INFO
unsigned short stack_low = 0xffff;
#endif

static void
idle_timer_handler (struct timer *t)
{
  unsigned long dt;

  dt = event_idle_counter * idle_loop_time;
  cpu_idle = dt;
  cpu_usage = 100 - (dt / 10000L);
  event_idle_counter = 0;
  timer_create (t, 1000, idle_timer_handler, TIMER_UNMASK);
}

static void
panel_putvalue (struct panel *p, lcd_line_t line, lcd_col_t col,
                unsigned int value, int need_all_digits)
{
  char c;

  c = (value / 100);
  if (c == 0 && need_all_digits == 0)
    c = ' ';
  else
    c = c + '0';

  panel_putchar (p, line, col, c);
  value = value % 100;
  col++;
  c = (value / 10);
  if (c == 0 && need_all_digits == 0)
    c = ' ';
  else
    c = c + '0';

  panel_putchar (p, line, col, c);
  c = (value % 10) + '0';
  panel_putchar (p, line, col + 1, c);
}

static unsigned long
ebcs_info_refresh (struct panel *p)
{
  char buf[32];

  switch (info_mode)
    {
    case INFO_VERSION:
    default:
      panel_putstring (p, 0, 0, "GEL v");
      panel_clear_line (p, 0, 6);
      panel_putstring (p, 0, 6, ebcs_version);
      panel_clear_line (p, 1, 0);
      panel_putstring (p, 1, 0, ebcs_date);
      break;

    case INFO_CPU_USAGE:
      panel_putstring (p, 0, 0, "CPU");
      panel_clear_line (p, 0, 3);
      panel_putvalue (p, 0, 4, cpu_usage, 0);
      panel_putstring (p, 0, 7, "%");
      panel_clear_line (p, 1, 4);
      sprintf (buf, "IDLE %ld us", cpu_idle);
      panel_putstring (p, 1, 0, buf);
      break;

#ifdef STACK_INFO
    case INFO_STACK_SIZE:
      panel_putstring (p, 0, 0, "Stack");
      panel_clear_line (p, 0, 5);
      panel_putvalue (p, 0, 7, 0x8000 - stack_low, 0);
      panel_clear_line (p, 1, 0);
      break;
#endif
    }
  return 1000L;
}

static void
ebcs_info_input (struct panel *p, enum key_mode mode, key_t key)
{
  if (mode == KEY_RELEASE && key == EBCS_KEY_NEXT)
    {
      info_mode ++;
      if (info_mode >= INFO_LAST)
        info_mode = INFO_FIRST;

      ebcs_info_refresh (p);
      return;
    }
  if (mode == KEY_RELEASE && key == EBCS_KEY_PREV)
    {
      panel_pop ();
      panel_push (&ebcs_var_panel);
      return;
    }
}

void
info_panel_initialize ()
{
  timer_create (&idle_timer, 1000, idle_timer_handler, TIMER_UNMASK);
  var_register (&idle_loop_var);
  panel_create (&ebcs_info_panel);
}

