/* main-panel.c -- Main panel 
   Copyright 2001 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <stddef.h>
#include <stdio.h>

#include <sys/locks.h>
#include <sys/ports.h>
#include <sys/sio.h>
#include <sys/interrupts.h>
#include <lcd.h>

#include <gel/timer.h>
#include <gel/buttons.h>
#include <gel/panel.h>
#include <gel/var.h>

#include <ebcs/ebcs.h>
#include <ebcs/triacs.h>
#include <ebcs/board.h>
#include <ebcs/temperature.h>
#include <ebcs/humidity.h>
#include <ebcs/main-panel.h>
#include <ebcs/date-panel.h>
#include <ebcs/info-panel.h>

#define EBCS_KEY_ON EBCS_KEY_PLUS
#define L_HUM  1
#define C_HUM  8
#define L_TEMP 1
#define C_TEMP 0

#define L_FAN_MODE 1
#define C_FAN_MODE 12

#define L_DAY   0
#define C_DAY   0
#define L_TIME  0
#define C_TIME  (7)

temperature_monitor_t temp_mon;
humidity_monitor_t humidity_mon;

enum fan_mode
{
  FAN_UNKNOWN = 0,
  FAN_AUTO,
  FAN_ON,
  FAN_OFF
};

static enum fan_mode fmode;

static void ebcs_main_input (struct panel *p, enum key_mode mode, key_t key);
static unsigned long ebcs_main_refresh (struct panel *p);

struct panel ebcs_main_panel = {
  0,
  0,
  ebcs_main_refresh,
  ebcs_main_input,
  0
};

unsigned short temp_factor = 1167;

static const struct var_def temp_factor_var =
{
  "Temperature factor",
  V_UINT16,
  F_INTEGER,
  500,
  3000,
  1167,
  { (unsigned char*) &temp_factor }
};

static void
ebcs_update_fan (struct panel *p)
{
  const char *s;
  
  switch (fmode)
    {
    case FAN_AUTO:
      s = "AUTO";
      break;

    case FAN_ON:
      s = "ON";
      break;

    case FAN_OFF:
      s = "OFF";
      break;

    default:
      s = "?";
      break;
    }
  panel_putitem (p, L_FAN_MODE, C_FAN_MODE, JUSTIFY_RIGHT, 4, s);
}

static unsigned long
ebcs_main_refresh (struct panel *p)
{
  struct timeval tv;
  struct timeval tv_end;
  struct tm t;
  char buf[32];

  if (fmode == FAN_UNKNOWN)
    {
      fmode = FAN_AUTO;
      ebcs_update_fan (p);
    }
  
  timer_gettime (&tv);
  localtime_r (&tv.tv_sec, &t);

  /* Update time.  */
  date_putday (p, L_DAY, C_DAY, &t);
  date_puttime (p, L_TIME, C_TIME, &t);
  
  sprintf (buf, "%ld%%",
           (long) (humidity_mon.cur_humidity / 10));
  panel_putitem (p, L_HUM, C_HUM, JUSTIFY_RIGHT, 3, buf);
  panel_putchar (p, L_HUM, C_HUM - 1, ' ');
  
  sprintf (buf, "%ld.%c\337C",
           (long) (temp_mon.temp_current / 10),
           (temp_mon.temp_current % 10) + '0');
  panel_putitem (p, L_TEMP, C_TEMP, JUSTIFY_RIGHT, 6, buf);  

  timer_gettime (&tv_end);
  if (tv.tv_sec != tv_end.tv_sec)
    return 10L;
  else
    return (1000000L - tv_end.tv_usec) / 1000L;
}

static void
ebcs_main_input (struct panel *p, enum key_mode mode, key_t key)
{
  if (mode == KEY_RELEASE && key == EBCS_KEY_NEXT)
    {
      panel_push (&ebcs_date_panel);
      return;
    }
  
  if (mode == KEY_RELEASE && key == EBCS_KEY_PREV)
    {
      panel_push (&ebcs_info_panel);
      return;
    }
  
  if (mode == KEY_PRESS && key == EBCS_KEY_ON)
    {
      switch (fmode)
        {
        case FAN_AUTO:
          fmode = FAN_ON;
          triac_set (TRIAC_FAN, TRIAC_ON);
          break;

        case FAN_ON:
          fmode = FAN_OFF;
          triac_set (TRIAC_FAN, TRIAC_OFF);
          break;

        case FAN_OFF:
        default:
          fmode = FAN_AUTO;
          break;
        }
      ebcs_update_fan (p);
    }
  if (mode == KEY_PRESS && key == EBCS_KEY_MINUS)
    {
      const char *state;
      
      if (triac_get (TRIAC_LIGHT) == TRIAC_ON)
        {
          triac_set (TRIAC_LIGHT, TRIAC_OFF);
          state = "OFF";
        }
      else
        {
          triac_set (TRIAC_LIGHT, TRIAC_ON);
          state = "ON";
        }
    }
}

void
main_panel_initialize ()
{
  humidity_initialize (&humidity_mon);
  temperature_initialize ();
  create_temperature_monitor (&temp_mon, M6811_ADR4, &temp_factor);

  panel_create (&ebcs_main_panel);
  panel_push (&ebcs_main_panel);
  fmode = FAN_UNKNOWN;

  var_register (&temp_factor_var);
}

