/* var-panel.c -- Control Variables panel 
   Copyright 2001 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <sys/locks.h>
#include <sys/ports.h>
#include <sys/sio.h>
#include <sys/interrupts.h>

#include <lcd.h>

#include <gel/timer.h>
#include <gel/buttons.h>
#include <gel/panel.h>
#include <gel/var.h>

#include <ebcs/ebcs.h>
#include <ebcs/board.h>
#include <ebcs/temperature.h>
#include <ebcs/humidity.h>
#include <ebcs/main-panel.h>
#include <ebcs/date-panel.h>
#include <ebcs/info-panel.h>
#include <ebcs/var-panel.h>

#define EBCS_KEY_ON EBCS_KEY_PLUS

static void ebcs_var_input (struct panel *p, enum key_mode mode, key_t key);
static unsigned long var_refresh (struct panel *p);
static void var_install (struct panel *p, const struct var_def *var);
static void var_enter (struct panel *p);
static void var_exit (struct panel *p);

static unsigned long value;
static const struct var_def *cur_var;
static unsigned cur_var_index;

struct panel ebcs_var_panel = {
  var_enter,
  var_exit,
  0,
  ebcs_var_input,
  0,
  10,
  0
};

static void
var_enter (struct panel *p)
{
  if (cur_var == 0)
    var_install (p, var_list[cur_var_index]);
}

static void
var_exit (struct panel *p)
{
  if (cur_var && var_value (cur_var) != value)
    {
      var_set_integer (cur_var, value);
    }
}

static void
var_install (struct panel *p, const struct var_def *var)
{
  if (cur_var && var_value (cur_var) != value)
    {
      var_set_integer (cur_var, value);
    }

  cur_var = var;
  value = var_value (var);
  panel_clear_line (p, 0, 0);
  panel_putstring (p, 0, 0, var->title);
  var_refresh (p);
}

static unsigned long
var_refresh (struct panel *p)
{
  char buf[32];

  panel_clear_line (p, 1, 0);
  var_format (buf, cur_var, value);
  panel_putstring (p, 1, 0, buf);
  return 1000L;
}

static void
ebcs_var_input (struct panel *p, enum key_mode mode, key_t key)
{
  if (mode == KEY_RELEASE && key == EBCS_KEY_NEXT)
    {
      cur_var_index++;
      if (cur_var_index >= var_count)
        cur_var_index = 0;

      var_install (p, var_list[cur_var_index]);
      return;
    }
  if (mode == KEY_RELEASE && key == EBCS_KEY_PREV)
    {
      panel_pop ();
      return;
    }
  if (mode == KEY_PRESS && key == EBCS_KEY_PLUS)
    {
      if (value < cur_var->max_value)
        value ++;

      var_refresh (p);
    }
  if (mode == KEY_PRESS && key == EBCS_KEY_MINUS)
    {
      if (value > cur_var->min_value)
        value --;

      var_refresh (p);
    }
}

void
var_panel_initialize ()
{
  panel_create (&ebcs_var_panel);
}

