/* buttons.c -- Control of GEL buttons
   Copyright 2001, 2002 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <sys/interrupts.h>
#include <sys/locks.h>
#include <sys/ports.h>
#include <gel/event.h>
#include <gel/timer.h>
#include <ebcs/ebcs.h>
#include <ebcs/remote.h>
#include <gel/buttons.h>

#ifndef UCHAR_MAX
# define UCHAR_MAX 255
#endif

/* The button hardware module uses an oscillator and 4 AND gates to
   apply the oscillator to the selected buttons.  The A7 input is
   connected to the buttons.  When a button is pressed and the oscillator
   was connected to it, the A7 input sees the oscillations.

   The oscillator uses 2 NOR gates.  It is enabled by applying a 0
   on one input of the NOR gates.  Applying a 1 forces the oscillator
   to 0.  The oscillator is not precise and oscillate arround 3khz.

   Buttons are scanned one at a time;  that is the oscillator is
   applied to only one of them at a time.  When it is applied to one
   button, we count the number of oscillation using the PACNT counter.
   The counter must reach a certain value to declare the button pressed.
   A button is polled for 8ms and then the next button is polled.
   After the last button is polled, the first button is polled again.

   A button is polled 3 times before deciding whether it is pressed or not.

  +----+----+----+----+---//----+
  | B1 | B2 | B3 | B4 |         |
  +----+----+----+----+---//----+
  <---->
    8ms
 (125Hz)
  <----------------------------->
    128ms

  fB = 3Khz => Pulses max = 24 (per 8ms)
                            96 (per 128ms)
  ..
  fB = 1Khz => Pulses max =  8 (per 8ms)
                            32 (per 128ms)

  Accumulator overflow cannot occur due to the sampling rate and the
  oscillator frequency.  If this occurs, this is either a hardware problem
  or a software error.  */
#define BUTTON_ENABLE  (0)
#define BUTTON_DISABLE (EBCS_BUTTON_ENABLE)

#define BUTTON_POLL_COUNT 3
#define NR_BUTTONS        4

static unsigned char cur_button PAGE0_ATTRIBUTE;
static unsigned long sample_timeout = 8;
static unsigned char button_sample_thresold = 8;
static struct timer sample_timer;
static button_mask_t button_state PAGE0_ATTRIBUTE;
static unsigned char button_sample[4];
static unsigned char update_counter PAGE0_ATTRIBUTE;

/* Acts as a pass-high filter.  There must be at least 8 pulses within
   8ms to take the counter into account.  */
static unsigned char button_freq_limit = 8;

struct event_def button_change_event;

static const unsigned char button_mask_table[] = {
  EBCS_BUTTON_EN_1 | BUTTON_ENABLE,
  EBCS_BUTTON_EN_2 | BUTTON_ENABLE,
  EBCS_BUTTON_EN_3 | BUTTON_ENABLE,
  EBCS_BUTTON_EN_4 | BUTTON_ENABLE
};

static void
button_remote_setup (unsigned char state)
{
  state |= 0x80;
  state &= ~(EBCS_LCD_E | EBCS_LCD_RS);
  remote_putc (~(state));
}

/* Called at 125 Hz.  */
static void
sample_handler (struct timer *t)
{
  unsigned short mask;
  unsigned char counter;

  /* Look how many pulses we got on PA7.  If there were enough, assume
     the current button is pressed.  One or several overflows on the
     accumulator pulse counter is treated like a button pressed.
     Accumulator overflow should never occur unless there is a hardware
     or software problem.  */
  mask = lock ();
  counter = _io_ports[M6811_PACNT];
  if (_io_ports[M6811_TFLG2] & M6811_PAOVF)
    counter = UCHAR_MAX / 4;
  restore (mask);

  if (counter >= button_freq_limit)
    {
      button_sample[cur_button] += counter;
    }
  else
    {
      button_sample[cur_button] >>= 1;
    }

  /* Move to the next button.  */
  cur_button++;
  if (cur_button > BUTTON_4)
    cur_button = BUTTON_1;

  /* Select the new button.  */
  button_remote_setup (button_mask_table[cur_button]);

  /* A new active button is now selected.  Clear the event counter
     and the overflow flag in case it was set.  */
  mask = lock ();
  _io_ports[M6811_PACNT] = 0;
  _io_ports[M6811_TFLG2] = M6811_PAOVF;
  restore (mask);

  /* Setup the timer to read the result of the new current button.  */
  timer_create (t, sample_timeout, sample_handler, TIMER_UNMASK);

  update_counter--;
  if (update_counter == 0)
    {
      int i;
      button_mask_t state = 0;

      update_counter = NR_BUTTONS * BUTTON_POLL_COUNT;

      /* Identify the buttons which are pressed based on
	 the number of pulses we got for them.  */
      for (i = BUTTON_1; i <= BUTTON_4; i++)
        {
          if (button_sample[i] >= button_sample_thresold)
            {
              state |= (1 << i);
            }
          button_sample[i] = 0;
        }

      /* Something changed, post an event.  */
      if (button_state != state)
        {
          button_state = state;
          event_post (&button_change_event, state);
        }
    }
}

/*! Return a mask for pressed buttons.

    @return the mask identifying pressed buttons
    @see button_enable, button_disable  */
button_mask_t
button_get_pressed (void)
{
  return button_state;
}
  
/*! Enable the polling of buttons.

    Enable the polling of buttons.  The button are scanned periodically
    and their state can be queried using \b button_get_pressed or
    by waiting for button events.

    The accumulator interrupts (PA7) are enabled and the button
    timer is activated.

    While buttons are polled, the application must not print on
    the LCD panel.  It is necessary to disable the polling.

    @see button_disable, button_get_pressed  */
void
button_enable (void)
{
  unsigned short mask;
  int i;

  for (i = 0; i < BUTTON_4; i++)
    button_sample[i] = 0;
  button_state = 0;
  update_counter = 4 * 3;

  /* Select the first button.  */
  cur_button = 0;
  button_remote_setup (button_mask_table[cur_button]);

  /* Restart the timer.  */
  timer_create (&sample_timer, sample_timeout, sample_handler, TIMER_UNMASK);

  /* Enable pulse accumulator interrupts.  */
  mask = lock ();
  _io_ports[M6811_TFLG2] = M6811_PAOVF;
  _io_ports[M6811_PACNT] = 0;

  _io_ports[M6811_TMSK2] &= ~M6811_PAOVI;

  /* Enable pulse accumulator.  */
  _io_ports[M6811_PACTL] |= M6811_PAEN;

  restore (mask);
}

/*! Disable the polling of buttons.

    Disable the polling of buttons.  This operation must be called
    before sending LCD commands.  Otherwise, these commands can
    disturb the button polling.

    The accumulator interrupts (PA7) are disabled and the button
    timers are deactivated.

    @see button_enable  */
void
button_disable (void)
{
  short mask;

  /* Disable pulse accumulator.  */
  mask = lock ();
  _io_ports[M6811_PACTL] &= ~M6811_PAEN;
  _io_ports[M6811_TMSK2] &= ~(M6811_PAOVI | M6811_PAII);
  restore (mask);

  /* Remove the pending timer.  */
  timer_remove (&sample_timer);
  button_remote_setup (BUTTON_DISABLE);
}

/*! Return true if the buttons are enabled.

    @return 1 if the buttons are enabled.
    @see button_enable, button_disable  */
int
button_is_enabled (void)
{
  return timer_is_active (&sample_timer);
}

/*! Initialize the button module.

    Configure the button module.  This function must be called
    once before any other.  After initialization, the button
    polling is disabled.

    @see button_enable, button_disable, button_get_pressed  */
void
button_initialize (void)
{
  /* Configure PA7 for input and event counting.  */
  _io_ports[M6811_PACTL] &= ~(M6811_DDRA7 | M6811_PAMOD | M6811_PAEN);
}
