/* humidity.c -- Humidity Measure
   Copyright 2001, 2002 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <sys/locks.h>
#include <sys/ports.h>
#include <sys/sio.h>
#include <sys/interrupts.h>
#include <gel/timer.h>
#include <gel/event.h>
#include <stddef.h>
#include <ebcs/ebcs.h>
#include <ebcs/board.h>
#include <ebcs/humidity.h>
#include <string.h>

/* Setup for a 8Mhz quartz, and prescaler set to 1
   (500ns period).  */
#define USEC_PER_TICK (1)
#define USEC_DIVISOR  (2)

#define MFREQ ((32*2)/4)
#define TICK_SUM (MFREQ * 0x10000)

void input_capture_1_interrupt (void);
void input_capture_2_interrupt (void);
void __attribute__((interrupt)) _input_capture_1_interrupt (void);
void __attribute__((interrupt)) _input_capture_2_interrupt (void);

struct event_def freq_update_event;

static event_callout freq_callout;

static unsigned short input_1_count PAGE0_ATTRIBUTE;
static unsigned short input_2_count PAGE0_ATTRIBUTE;
static unsigned short m_count_1 PAGE0_ATTRIBUTE;
static unsigned short m_count_2 PAGE0_ATTRIBUTE;


void __attribute__((interrupt))
_input_capture_1_interrupt (void)
{
  asm volatile (".globl input_capture_1_interrupt\n"
                "input_capture_1_interrupt:");
  input_1_count++;
  _io_ports[M6811_TFLG1] = M6811_IC1F;
  asm volatile ("rti");
}

void __attribute__((interrupt))
_input_capture_2_interrupt (void)
{
  asm volatile (".globl input_capture_1_interrupt\n"
                "input_capture_2_interrupt:");
  input_2_count++;
  _io_ports[M6811_TFLG1] = M6811_IC2F;
  asm volatile ("rti");
}

static void
timer_overflow (void)
{
  static unsigned char nr_overflow = MFREQ;

  nr_overflow--;
  if (nr_overflow == 0)
    {
      m_count_1 = input_1_count;
      m_count_2 = input_2_count;
      input_1_count = 0;
      input_2_count = 0;
      nr_overflow = MFREQ;
      event_post (&freq_update_event, 0);
    }
}

static unsigned short
compute_humidity (unsigned long dt, unsigned short cnt)
{
  unsigned long h;
  
  if (cnt == 0)
    return 0;

  if (cnt)
    dt = dt / cnt;
  dt = (dt * USEC_PER_TICK) / USEC_DIVISOR;
  dt = 1000000000L / dt;
  dt = dt / 1000;
  h = 10560000 / (dt * 16);
  h -= 1000;
  h *= 100;
  h /= 80;
  return h;
}

static void
freq_event_handler (event_callout *callout, event *ev)
{
  humidity_monitor_t *mon = (humidity_monitor_t*) callout->data;

  mon->cur_humidity = compute_humidity (TICK_SUM, m_count_2);
}

/*! Initialize humidity module.

    Setup the humidity module to measure the frequency on IC1 and IC2
    and report the humidity.  */
void
humidity_initialize (humidity_monitor_t *mon)
{
  memset (mon, 0, sizeof (mon[0]));
  set_interrupt_handler (TIMER_INPUT1_VECTOR, input_capture_1_interrupt);
  set_interrupt_handler (TIMER_INPUT2_VECTOR, input_capture_2_interrupt);
  set_timer_overflow_handler (timer_overflow);
  _io_ports[M6811_TCTL2] = M6811_EDG1B | M6811_EDG2B | M6811_EDG3B;
  _io_ports[M6811_TMSK1] = M6811_IC1I | M6811_IC2I;

  event_register (&freq_update_event, &freq_callout, freq_event_handler, mon);
}

