/* lcd.c -- LCD Control Panel Operations
   Copyright 2000, 2001 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <sys/param.h>
#include <sys/locks.h>
#include <sys/ports.h>
#include <sys/sio.h>
#include <sys/spi.h>
#include <ebcs/ebcs.h>
#include <lcd.h>
#include <time.h>
#include <ebcs/remote.h>

#define LCD_DEFAULT (unsigned char) (0x80)
#define LCD_V(X) (unsigned char) ((~ (X) & ~(LCD_RS|LCD_E)))

static const unsigned char
lcd_byte_translator[16] = {
  /* 0000 */ LCD_V (LCD_DEFAULT | 0),
  /* 0001 */ LCD_V (LCD_DEFAULT | LCD_BIT_4),
  /* 0010 */ LCD_V (LCD_DEFAULT | LCD_BIT_5),
  /* 0011 */ LCD_V (LCD_DEFAULT | LCD_BIT_5 | LCD_BIT_4),
  /* 0100 */ LCD_V (LCD_DEFAULT | LCD_BIT_6),
  /* 0101 */ LCD_V (LCD_DEFAULT | LCD_BIT_6 | LCD_BIT_4),
  /* 0110 */ LCD_V (LCD_DEFAULT | LCD_BIT_6 | LCD_BIT_5),
  /* 0111 */ LCD_V (LCD_DEFAULT | LCD_BIT_6 | LCD_BIT_5 | LCD_BIT_4),
  /* 1000 */ LCD_V (LCD_DEFAULT | LCD_BIT_7),
  /* 1001 */ LCD_V (LCD_DEFAULT | LCD_BIT_7 | LCD_BIT_4),
  /* 1010 */ LCD_V (LCD_DEFAULT | LCD_BIT_7 | LCD_BIT_5),
  /* 1011 */ LCD_V (LCD_DEFAULT | LCD_BIT_7 | LCD_BIT_5 | LCD_BIT_4),
  /* 1100 */ LCD_V (LCD_DEFAULT | LCD_BIT_7 | LCD_BIT_6),
  /* 1101 */ LCD_V (LCD_DEFAULT | LCD_BIT_7 | LCD_BIT_6 | LCD_BIT_4),
  /* 1110 */ LCD_V (LCD_DEFAULT | LCD_BIT_7 | LCD_BIT_6 | LCD_BIT_5),
  /* 1111 */ LCD_V (LCD_DEFAULT | LCD_BIT_7 | LCD_BIT_6
                    | LCD_BIT_5 | LCD_BIT_4)
};

static unsigned char lcd_mode;

/* Send 4-bits to the LCD controller.  */
static void
lcd_send_4bits (unsigned char val, unsigned char mode)
{
  val = val & 0x0F;

  val = lcd_byte_translator[val];
  if (!mode)
    val |= LCD_RS;
  remote_putc (val);
  val |= LCD_E;
  remote_putc (val);
}

static void
lcd_send (unsigned char val, unsigned char mode)
{
  lcd_send_4bits (val >> 4, mode);
  lcd_send_4bits (val, mode);
}

/** Reset the LCD panel.

    The SPI must be initialized before trying to reset the LCD controller.

    This operation can take a few milliseconds due to delay constraints
    imposed by the HD44780 controller.
*/
void
lcd_reset (void)
{
  remote_initialize ();

  /* Init sequence from Hitachi document.  */
  lcd_send_4bits (0x3, LCD_RS);
  udelay (4100);

  lcd_send_4bits (0x3, LCD_RS);
  udelay (200);

  lcd_send_4bits (0x3, LCD_RS);
  udelay (200);

  lcd_send_4bits (0x2, 0);
  udelay (2000);

  /* LCD 4-bits, 2 lines.  */
  lcd_send (0x28, 0);

  lcd_send (0x08, 0);
  lcd_send (0x01, 0);
  udelay (2000);

  lcd_mode = 0x0f;
  lcd_send (0x0F, 0);
  lcd_send (0x06, 0);
}

void
lcd_home (void)
{
  lcd_send (0x02, 0);
  udelay (2000);
}

void
lcd_clear (void)
{
  lcd_send (0x01, 0);
  udelay (2000);
}

static void
lcd_set_mode (unsigned char mask, int mode)
{
  unsigned char new_mode;

  mask &= 0x07;
  new_mode = lcd_mode | mask;
  new_mode |= 0x08;
  if (mode == 0)
    new_mode &= ~mask;
  if (new_mode != lcd_mode || 1)
    {
      lcd_mode = new_mode;
      lcd_send (new_mode, 0);
    }
}

void
lcd_blink (int mode)
{
  lcd_set_mode (0x01, mode);
}

void
lcd_cursor (int mode)
{
  lcd_set_mode (0x02, mode);
}

void
lcd_display (int mode)
{
  lcd_set_mode (0x04, mode);
}

void
lcd_shift (int direction)
{
  if (direction)
    lcd_send (0x1c, 0);
  else
    lcd_send (0x18, 0);
}

/*! Print the string on the LCD panel.

  Print the string \i p at the current cursor location on the LCD panel.
  The cursor is automatically moved to the end of the string.

  @see lcd_goto  */
void
lcd_print (const char *p)
{
  while (*p)
    {
      lcd_send (*p++, LCD_RS);
    }
}

/*! Print the string on the LCD panel.

  Print the string \i p at the current cursor location on the LCD panel.
  The cursor is automatically moved to the end of the string.

  @see lcd_goto  */
void
lcd_putchar (unsigned char c)
{
  lcd_send (c, LCD_RS);
}

/*! Move the cursor to the specified location.

  Move the cursor to the \i col and \i line on the LCD panel.

  @see lcd_print  */
void
lcd_goto (lcd_col_t col, lcd_line_t line)
{
  unsigned char addr;

  addr = col & 0x3F;
  if (line)
    addr |= 0x40;

  addr |= 0x80;
  lcd_send (addr, 0);
}

/*! Print the string at a given location on the LCD panel.

  Move the cursor to \i col and \i line and print the string \i msg.

  @see lcd_goto, lcd_print  */
void
lcd_print_at (lcd_col_t col, lcd_line_t line, const char *msg)
{
  lcd_goto (col, line);
  lcd_print (msg);
}


