/* triacs.c -- Triacs Control Interface
   Copyright 2001 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <sys/locks.h>
#include <sys/ports.h>
#include <ebcs/ebcs.h>
#include <ebcs/triacs.h>
#include <assert.h>

/*! Table of triac events.

  There is one event for each triac.  The event is triggered when the
  state of the triac is changed.

  @see triac_set, triac_get, event_post
*/
static struct event_def triac_light_event;
static struct event_def triac_fan_event;
static unsigned char triac_states;

event_type triac_events[] = {
  &triac_light_event,
  &triac_fan_event
};

triac_stat_t triac_stats[2];

static inline unsigned char
triac_get_mask (triac_id_t id)
{
  switch (id)
    {
    case TRIAC_LIGHT:
      return EBCS_CMD_LIGHT;

    case TRIAC_FAN:
      return EBCS_CMD_FAN;

    default:
      assert (id == TRIAC_LIGHT || id == TRIAC_FAN);
      return 0;
    }
}

/*! Update the triac outputs.

  The triac state has changed.  Setup the new outputs and post an
  event on the triac that changed.  */
static void
triac_update (triac_id_t id)
{
  unsigned char val;
  unsigned short mask;

  /* Protect in case the PORT A is changed under interrupts.  */
  mask = lock ();
  val = (_io_ports[M6811_PORTA] & ~(EBCS_CMD_LIGHT | EBCS_CMD_FAN));
  val |= triac_states;
  _io_ports[M6811_PORTA] = val;
  restore (mask);

  event_post (triac_events[id], 0);
}

/*! Turn on or off the given triac.

  @param id the triac identification
  @param state the new state of the triac (ON or OFF)

  @see triac_get
 */
void
triac_set (triac_id_t id, triac_state_t state)
{
  unsigned char mask;
  triac_stat_t *stat;

  stat = &triac_stats[id];
  mask = triac_get_mask (id);
  if (state == TRIAC_ON)
    {
      if ((triac_states & mask) == 0)
	{
	  triac_states |= mask;
	  triac_update (id);
	  stat->on_counts++;
	  timer_gettime (&stat->on_time);
	}
    }
  else
    {
      if ((triac_states & mask) != 0)
	{
	  struct timeval tv;

	  triac_states &= ~mask;
	  triac_update (id);
	  timer_gettime (&tv);
	  timevalsub (&tv, &stat->on_time);
	  timevaladd (&stat->on_total_time, &tv);
	}
    }
}

/*! Get the current state of the triac.

  @param id the triac identification
  @see triac_set
*/
triac_state_t
triac_get (triac_id_t id)
{
  unsigned char mask;

  mask = triac_get_mask (id);
  return (triac_states & mask) ? TRIAC_ON : TRIAC_OFF;
}
